package gs.smolban.model.users

/** Roles define what each [[User]] is allowed to do.
  *
  * @param name
  *   The name of the role.
  * @param scope
  *   The [[Scope]] to which the role applies.
  */
sealed abstract class Role(
  val name: String,
  val scope: Scope
):

  override def toString(): String =
    s"[$name]${Role.Delimiter}$scope"

object Role:

  val Delimiter: String = ":"

  /** Administrator for the API. Can perform any operation through the API.
    */
  case object ApiAdmin extends Role(s"api${Delimiter}admin", Scope.ApiGlobal)

  /** Global read access to the API.
    */
  case object ApiGlobalReader
      extends Role(s"api${Delimiter}reader", Scope.ApiGlobal)

  /** Global write access to the API.
    */
  case object ApiGlobalWriter
      extends Role(s"api${Delimiter}writer", Scope.ApiGlobal)

  /** Administrator for a specific [[gs.smolban.model.Group]] via the API. Can
    * perform any operation through the API for the indicated group.
    *
    * @param groupId
    *   The unique identifier for the group to which this role is scoped.
    */
  case class ApiGroupAdmin(
    groupId: gs.smolban.model.Group.Id
  ) extends Role(s"api${Delimiter}group_admin", Scope.ApiGroup(groupId))

  /** Grants API read access for a specific [[gs.smolban.model.Group]].
    *
    * @param groupId
    *   The unique identifier for the group to which this role is scoped.
    */
  case class ApiGroupReader(
    groupId: gs.smolban.model.Group.Id
  ) extends Role(s"api${Delimiter}group_reader", Scope.ApiGroup(groupId))

  /** Grants API write access for a specific [[gs.smolban.model.Group]].
    *
    * @param groupId
    *   The unique identifier for the group to which this role is scoped.
    */
  case class ApiGroupWriter(
    groupId: gs.smolban.model.Group.Id
  ) extends Role(s"api${Delimiter}group_writer", Scope.ApiGroup(groupId))

  /** Administrator for the UI. Can perform any operation through the UI.
    */
  case object UiAdmin extends Role(s"ui${Delimiter}admin", Scope.UiGlobal)

  /** Global read access to the UI.
    */
  case object UiGlobalReader extends Role("ui:reader", Scope.UiGlobal)

  /** Global write access to the UI.
    */
  case object UiGlobalWriter extends Role("ui:writer", Scope.UiGlobal)

  /** Administrator for a specific [[gs.smolban.model.Group]] via the UI. Can
    * perform any operation through the UI for the indicated group.
    *
    * @param groupId
    *   The unique identifier for the group to which this role is scoped.
    */
  case class UiGroupAdmin(
    groupId: gs.smolban.model.Group.Id
  ) extends Role(s"ui${Delimiter}group_admin", Scope.UiGroup(groupId))

  /** Grants UI read access for a specific [[gs.smolban.model.Group]].
    *
    * @param groupId
    *   The unique identifier for the group to which this role is scoped.
    */
  case class UiGroupReader(
    groupId: gs.smolban.model.Group.Id
  ) extends Role(s"ui${Delimiter}group_reader", Scope.UiGroup(groupId))

  /** Grants UI write access for a specific [[gs.smolban.model.Group]].
    *
    * @param groupId
    *   The unique identifier for the group to which this role is scoped.
    */
  case class UiGroupWriter(
    groupId: gs.smolban.model.Group.Id
  ) extends Role(s"ui${Delimiter}group_writer", Scope.UiGroup(groupId))

end Role
